/* ----------------------------------------------------------------------- *
 * G L S a m p l e 3 V i e w . c p p
 *
 * implementation of the CGLSample3View class
 *
 * Copyright 1996 by Interface Technologies, Inc. All Rights Reserved.
 * GLSample3 Authored by N. Alan Oursland
 * ----------------------------------------------------------------------- */

#include "stdafx.h"
#include "GLSample3.h"

#include "GLSample3Doc.h"
#include "GLSample3View.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CGLSample3View

IMPLEMENT_DYNCREATE(CGLSample3View, CView)

BEGIN_MESSAGE_MAP(CGLSample3View, CView)
	//{{AFX_MSG_MAP(CGLSample3View)
	ON_WM_CREATE()
	ON_WM_DESTROY()
	ON_WM_SIZE()
	ON_WM_PAINT()
	ON_WM_LBUTTONUP()
	ON_WM_LBUTTONDOWN()
	ON_WM_RBUTTONUP()
	ON_WM_RBUTTONDOWN()
	ON_WM_MOUSEMOVE()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CGLSample3View construction/destruction

CGLSample3View::CGLSample3View()
{
	m_hGLContext = NULL;
	m_GLPixelIndex = 0;
	m_RightButtonDown = FALSE;
	m_LeftButtonDown = FALSE;
}

CGLSample3View::~CGLSample3View()
{
}

BOOL CGLSample3View::PreCreateWindow(CREATESTRUCT& cs)
{
	cs.style |= (WS_CLIPCHILDREN | WS_CLIPSIBLINGS);

	return CView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CGLSample3View drawing

void CGLSample3View::OnDraw(CDC* pDC)
{
	CGLSample3Doc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	// TODO: add draw code for native data here
}

/////////////////////////////////////////////////////////////////////////////
// CGLSample3View diagnostics

#ifdef _DEBUG
void CGLSample3View::AssertValid() const
{
	CView::AssertValid();
}

void CGLSample3View::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CGLSample3Doc* CGLSample3View::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CGLSample3Doc)));
	return (CGLSample3Doc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CGLSample3View message handlers

BOOL CGLSample3View::SetWindowPixelFormat(HDC hDC)
{
	PIXELFORMATDESCRIPTOR pixelDesc;

	pixelDesc.nSize		= sizeof(PIXELFORMATDESCRIPTOR);
	pixelDesc.nVersion	= 1;

	pixelDesc.dwFlags	=	PFD_DRAW_TO_WINDOW | 
//					PFD_DRAW_TO_BITMAP | 
					PFD_SUPPORT_OPENGL | 
//					PFD_SUPPORT_GDI | 
					PFD_DOUBLEBUFFER  | 
					PFD_STEREO_DONTCARE;

	pixelDesc.iPixelType		= PFD_TYPE_RGBA;
	pixelDesc.cColorBits		= 32;
	pixelDesc.cRedBits		= 8;
	pixelDesc.cRedShift		= 16;
	pixelDesc.cGreenBits		= 8;
	pixelDesc.cGreenShift		= 8;
	pixelDesc.cBlueBits		= 8;
	pixelDesc.cBlueShift		= 0;
	pixelDesc.cAlphaBits		= 0;
	pixelDesc.cAlphaShift		= 0;
	pixelDesc.cAccumBits		= 64;	
	pixelDesc.cAccumRedBits		= 16;
	pixelDesc.cAccumGreenBits	= 16;
	pixelDesc.cAccumBlueBits	= 16;
	pixelDesc.cAccumAlphaBits	= 0;
	pixelDesc.cDepthBits		= 32;
	pixelDesc.cStencilBits		= 8;
	pixelDesc.cAuxBuffers		= 0;
	pixelDesc.iLayerType		= PFD_MAIN_PLANE;
	pixelDesc.bReserved		= 0;
	pixelDesc.dwLayerMask		= 0;
	pixelDesc.dwVisibleMask		= 0;
	pixelDesc.dwDamageMask		= 0;

	m_GLPixelIndex = ChoosePixelFormat( hDC, &pixelDesc);
	if (m_GLPixelIndex==0) // Let's choose a default index.
	{
		m_GLPixelIndex = 1;	
		if (DescribePixelFormat(hDC, 
						m_GLPixelIndex, 
						sizeof(PIXELFORMATDESCRIPTOR), 
						&pixelDesc)==0)
		{
			return FALSE;
		}
	}

	if (SetPixelFormat( hDC, 
				  m_GLPixelIndex, 
				  &pixelDesc)==FALSE)
	{
		return FALSE;
	}

	return TRUE;
}

int CGLSample3View::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CView::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	HWND hWnd = GetSafeHwnd();
	HDC hDC = ::GetDC(hWnd);

	if (SetWindowPixelFormat(hDC)==FALSE)
		return 0;
	
	if (CreateViewGLContext(hDC)==FALSE)
		return 0;
	
	return 0;
}

BOOL CGLSample3View::CreateViewGLContext(HDC hDC)
{
	m_hGLContext = wglCreateContext(hDC);
	if (m_hGLContext == NULL)
	{
		return FALSE;
	}

	if (wglMakeCurrent(hDC, m_hGLContext)==FALSE)
	{
		return FALSE;
	}

	return TRUE;
}

void CGLSample3View::OnDestroy() 
{
	if(wglGetCurrentContext()!=NULL) 
	{
		// make the rendering context not current
		wglMakeCurrent(NULL, NULL) ;
	}
	
	if (m_hGLContext!=NULL)
	{
		wglDeleteContext(m_hGLContext);
		m_hGLContext = NULL;
	}

	// Now the associated DC can be released.
	CView::OnDestroy(); 
}

void CGLSample3View::OnSize(UINT nType, int cx, int cy) 
{
	CView::OnSize(nType, cx, cy);
	
	
	GLsizei width, height;
	GLdouble aspect;

	width = cx;
	height = cy;

	if (cy==0)
		aspect = (GLdouble)width;
	else
		aspect = (GLdouble)width/(GLdouble)height;

	glViewport(0, 0, width, height);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluOrtho2D(0.0, 500.0*aspect, 0.0, 500.0);
	glMatrixMode(GL_MODELVIEW);

	glDrawBuffer(GL_BACK);
}

void CGLSample3View::OnPaint() 
{
	CPaintDC dc(this); // device context for painting
	
	CGLSample3Doc* pDoc = GetDocument();
	pDoc->RenderScene();

	if (SwapBuffers(dc.m_ps.hdc)==FALSE)
	{
			DWORD errorCode = GetLastError();
			CString errorStr;
			errorStr.Format("SwapBuffers returned error code %d.", errorCode);
			AfxMessageBox(errorStr);
			return;
	}

}

void CGLSample3View::OnLButtonUp(UINT nFlags, CPoint point) 
{
	m_LeftButtonDown = FALSE;
	CView::OnLButtonUp(nFlags, point);
}

void CGLSample3View::OnLButtonDown(UINT nFlags, CPoint point) 
{
	m_LeftButtonDown = TRUE;
	m_LeftDownPos = point;
	CView::OnLButtonDown(nFlags, point);
}

void CGLSample3View::OnRButtonUp(UINT nFlags, CPoint point) 
{
	m_RightButtonDown = FALSE;
	CView::OnRButtonUp(nFlags, point);
}

void CGLSample3View::OnRButtonDown(UINT nFlags, CPoint point) 
{
	m_RightButtonDown = TRUE;
	m_RightDownPos = point;
	CView::OnRButtonDown(nFlags, point);
}

void CGLSample3View::OnMouseMove(UINT nFlags, CPoint point) 
{
	if (m_RightButtonDown)
	{
		CGLSample3Doc* pDoc = GetDocument();
		CSize rotate = m_RightDownPos - point;
		m_RightDownPos = point;

		pDoc->m_angle1 += rotate.cx/3;
		pDoc->m_angle2 += rotate.cy/3;
		InvalidateRect(NULL, FALSE);
	}

	if (m_LeftButtonDown)
	{
		CGLSample3Doc* pDoc = GetDocument();
		CSize translate = m_LeftDownPos - point;
		m_LeftDownPos = point;
		pDoc->m_transX -= translate.cx/3;
		pDoc->m_transY += translate.cy/3;
		InvalidateRect(NULL, FALSE);
	}
	
	CView::OnMouseMove(nFlags, point);
}
