
' ####################
' #####  PROLOG  #####
' ####################

' A function to create multiple directories from a local pathspec

VERSION "0.0001"
CONSOLE

IMPORT "xst"
IMPORT "xsx"

DECLARE FUNCTION Entry ()
DECLARE FUNCTION MakeMultiFolders (path$)

$$MMF_SUCCESS                =  0
$$MMF_INVALID_PATH           = -1
$$MMF_PATH_CONTAINS_FILESPEC = -2
$$MMF_INVALID_DRIVESPEC      = -3
$$MMF_XSTMAKEDIRECTORY_ERROR = -4
$$MMF_INVALID_ATTRIBUTE      = -5


' ######################
' #####  Entry ()  #####
' ######################

FUNCTION Entry ()

	path$ = ""
	err = MakeMultiFolders (path$)
	? "err = "; err

	path$ = "d:"
	err = MakeMultiFolders (path$)
	? "err = "; err

	path$ = "d:\\"
	err = MakeMultiFolders (path$)
	? "err = "; err

	path$ = "q:\\one\\two\\"
	err = MakeMultiFolders (path$)
	? "err = "; err

	path$ = "d:\\one\\two\\three\\four\\myfile.txt"
	err = MakeMultiFolders (path$)
	? "err = "; err

	path$ = "\\one\\two\\three\\four\\"
	err = MakeMultiFolders (path$)
	? "err = "; err

	path$ = "d:\\one\\two\\three\\four\\"
	err = MakeMultiFolders (path$)
	? "err = "; err

	path$ = "d:\\one\\two\\three\\four"
	err = MakeMultiFolders (path$)
	? "err = "; err

  a$ = INLINE$ ("Press Enter to quit >")

END FUNCTION
'
' ##############################
' #####  MakeMultiFolders  #####
' ##############################
'
' MakeMultiFolders ("d:\\one\\two\\three\\four\\")
'
FUNCTION MakeMultiFolders (path$)

	IFZ path$ THEN RETURN $$MMF_INVALID_PATH

	' is absolute path
	IFZ XstIsAbsolutePath(path$) THEN RETURN $$MMF_INVALID_PATH

	' is ther a filename?
	n = RINSTR(path$, "\\")
	m = RINSTR(path$, ".")
	IF (n > 0) && (m > 0) THEN
		IF m > n THEN
			' period found after last backslash
			RETURN $$MMF_PATH_CONTAINS_FILESPEC
		END IF
	END IF

	' is there a drive spec?
	n = INSTR(path$, ":")
	IFZ n THEN RETURN $$MMF_INVALID_PATH

	drive$ = LEFT$(path$, n)
	dir$ = MID$ (path$, n+2)

	' are there folders?
	IFZ dir$ THEN RETURN $$MMF_INVALID_PATH

	' does drive exist?
	XstGetFileAttributes(drive$ , @attributes)
	IFZ attributes THEN RETURN $$MMF_INVALID_DRIVESPEC

	' get rid of a leading and trailing backslashs
	IF RIGHT$(dir$) = "\\" THEN dir$ = LEFT$(dir$, LEN(dir$) - 1)
	IF LEFT$(dir$) = "\\" THEN dir$ = RIGHT$(dir$, LEN(dir$) - 1)

	' split folders into an array
	XstParseStringToStringArray(dir$, "\\", @dir$[])
	dirSpec$ = drive$

	upp = UBOUND(dir$[])

	FOR i = 0 TO upp
		dirSpec$ = dirSpec$ + "\\" + dir$[i]

		' does it already exist?
		XstGetFileAttributes(dirSpec$, @attributes)
		IF attributes THEN DO NEXT

		' create directory
		IF XstMakeDirectory (dirSpec$) THEN RETURN $$MMF_XSTMAKEDIRECTORY_ERROR

		XstGetFileAttributes(dirSpec$, @attributes)
		IF attributes <> $$FileDirectory THEN RETURN $$MMF_INVALID_ATTRIBUTE

	NEXT i

END FUNCTION
END PROGRAM