'
' ####################
' #####  PROLOG  #####
' ####################
'
' ping demo using IcmpSendEcho() function in icmp.dll.
' example use: ping www.allapi.net
'
VERSION "0.0001"
CONSOLE
'
	IMPORT "xst_s"
	IMPORT "xsx_s"
	IMPORT "wsock32"
	IMPORT "kernel32"

TYPE ICMP_HEADER
  UBYTE .type
  UBYTE .code
  USHORT .checksum
END TYPE

TYPE IP_OPTION_INFORMATION
  UBYTE .Ttl
  UBYTE .Tos
  UBYTE .Flags
  ULONG .OptionsSize
  STRING * 128 .OptionsData
END TYPE

TYPE ICMP_ECHO_REPLY
  IN_ADDR .Address
  ULONG .Status
  ULONG .RoundTripTime
  USHORT .DataSize
	USHORT .Reserved
  XLONG .Data
  IP_OPTION_INFORMATION .Options
END TYPE


DECLARE FUNCTION Entry ()
DECLARE FUNCTION WSAInit ()
DECLARE FUNCTION GetIPAddr (IPName$, @numIPAddr$)
DECLARE FUNCTION Ping (sAddr$, data$, ICMP_ECHO_REPLY echo)

$$IP_SUCCESS = 0
$$IP_STATUS_BASE = 11000
$$IP_BUF_TOO_SMALL = 11001
$$IP_DEST_NET_UNREACHABLE = 11002
$$IP_DEST_HOST_UNREACHABLE = 11003
$$IP_DEST_PROT_UNREACHABLE = 11004
$$IP_DEST_PORT_UNREACHABLE = 11005
$$IP_NO_RESOURCES = 11006
$$IP_BAD_OPTION = 11007
$$IP_HW_ERROR = 11008
$$IP_PACKET_TOO_BIG = 11009
$$IP_REQ_TIMED_OUT = 11010
$$IP_BAD_REQ = 11011
$$IP_BAD_ROUTE = 11012
$$IP_TTL_EXPIRED_TRANSIT = 11013
$$IP_TTL_EXPIRED_REASSEM = 11014
$$IP_PARAM_PROBLEM = 11015
$$IP_SOURCE_QUENCH = 11016
$$IP_OPTION_TOO_BIG = 11017
$$IP_BAD_DESTINATION = 11018
$$IP_ADDR_DELETED = 11019
$$IP_SPEC_MTU_CHANGE = 11020
$$IP_MTU_CHANGE = 11021
$$IP_UNLOAD = 11022
$$IP_ADDR_ADDED = 11023
$$IP_GENERAL_FAILURE = 11050
$$MAX_IP_STATUS = 11050
$$IP_PENDING = 11255

$$PING_TIMEOUT 	= 2000
'
'
' ######################
' #####  Entry ()  #####
' ######################
'
FUNCTION Entry ()

	ICMP_ECHO_REPLY echo

	XstGetCommandLineArguments (@argc, @argv$[])

	IF (argc != 2) THEN
    ? "Usage:\t ping [address]"
	?
	? INLINE$ ("Press any key to quit > ")
    RETURN 1
  END IF

	WSAInit ()
	ret = Ping (argv$[1], "echo", @echo)

  IF ret = $$IP_SUCCESS THEN ? "ping -> success"
	? "ping -> Status        ="; echo.Status
	x = echo.Address
	addr$ = STRING$(x{8,0}) + "." + STRING$(x{8,8}) + "." + STRING$(x{8,16}) + "." + STRING$(x{8,24})
	? "ping -> Address       = "; addr$
	? "ping -> RoundTripTime ="; echo.RoundTripTime; " ms"
	? "ping -> DataSize      ="; echo.DataSize; " bytes"

	WSACleanup ()
	?
  a$ = INLINE$ ("Press Enter to quit >")

	RETURN exit

END FUNCTION

FUNCTION WSAInit ()

	WSADATA wsadata

	version = 2 OR (2 << 8)											' version winsock v2.2 (MAKEWORD (2,2))
	ret = WSAStartup (version, &wsadata)				' initialize winsock.dll
	IF ret THEN
		PRINT "ping -> WSAStartup error:"; ret
		WSACleanup ()
		RETURN
	END IF

	IF wsadata.wVersion != version THEN
		PRINT "ping -> winsock version not supported"
		WSACleanup ()
		RETURN
	END IF

END FUNCTION

FUNCTION Ping (sAddr$, data$, ICMP_ECHO_REPLY echo)

  HOSTENT host
	IN_ADDR inaddr

	sAddr$ = sAddr$ + NULL$(64 - LEN(sAddr$))

	' have we got a numeric IP address
	inaddr = inet_addr (&sAddr$)

	IF (inaddr == $$INADDR_NONE) THEN
		GetIPAddr (sAddr$, @numIPAddr$)
		inaddr = inet_addr (&numIPAddr$)
	END IF

	IF inaddr = $$INADDR_NONE THEN
		PRINT "ping -> inet_addr: bad IP format"
		RETURN $$TRUE
	END IF

	DIM args[]
	icmphandle = XstCall ("IcmpCreateFile", "icmp.dll", @args[])

	IFZ icmphandle THEN
		PRINT "ping -> IcmpCreateFile: error"
		RETURN $$TRUE
	END IF

	DIM args[7]
	args[0] = icmphandle
	args[1] = inaddr
	args[2] = &data$
	args[3] = LEN(data$)
	args[4] = NULL
	args[5] = &echo
	args[6] = SIZE(echo) + 8
	args[7] = $$PING_TIMEOUT
	ret = XstCall ("IcmpSendEcho", "icmp.dll", @args[])

	DIM args[0]
	args[0] = icmphandle
	XstCall ("IcmpCloseHandle", "icmp.dll", @args[])

	RETURN echo.Status

END FUNCTION

FUNCTION GetIPAddr (IPName$, @numIPAddr$)
	WSADATA wsadata
	HOSTENT	host

	host = gethostbyname (&IPName$)

	IF host.h_addr_list <> 0 THEN
		addr = 0
		RtlMoveMemory (&addr, host.h_addr_list, 4)
		RtlMoveMemory (&addr, addr, 4)

		addr2 = inet_ntoa (addr)

		numIPAddr$ = NULL$ (512)
		RtlMoveMemory (&numIPAddr$, addr2, LEN(numIPAddr$))
		numIPAddr$ = CSIZE$ (numIPAddr$)
		IFZ numIPAddr$ THEN exit = $$TRUE
	ELSE
		exit = $$TRUE
	END IF

	RETURN exit

END FUNCTION

END PROGRAM