

' Basic implementation of HTTP protocol
' Ken Minogue, Jan 2008
'
' ####################
' #####  PROLOG  #####
' ####################
'
PROGRAM	"xinhttp"
VERSION	"1.0"
CONSOLE

IMPORT	"xst"
IMPORT  "xsx"
IMPORT  "xin"
IMPORT  "xio"

DECLARE FUNCTION  Entry ()
DECLARE FUNCTION  HTTP (URL$, referer$, callback, header$, content$)
DECLARE FUNCTION  Blowback ()
DECLARE FUNCTION  Callback (bytes)
'
'
' ######################
' #####  Entry ()  #####
' ######################
'
FUNCTION  Entry ()

	XioSetConsoleBufferSize (XioGetStdOut(), 0, 2000) 
'	XinSetDebug ($$TRUE)

'	URL$ = "http://www.livedepartureboards.co.uk/ldb/station.aspx?T=MAN"
	URL$ = "www.irate.org.uk/index.htm"
	HTTP (@URL$, "", &Callback(), @header$, @content$)

	PRINT "*******   Header  ********"
	PRINT header$
	PRINT

	PRINT "*******  Content  ********"
	PRINT content$

	XinSetDebug($$FALSE)
	
	INLINE$ ("Press any key to quit")

END FUNCTION

'
'
' #####################
' #####  HTTP ()  #####
' #####################
'
'Input
'  URL$ is the address of the web page to retrieve, with or without the "http://" part.
'  referer$ is usually not needed; some sites will not allow a file to be downloaded unless the referer
'    (the URL of the page from which the request is made) is specified.
'  callback is the address of an optional function to be called periodically while attempting to establish
'    a connection and while receiving data.  The function can do any processing necessary, such as calling
'    XgrProcessMessages() to keep the GUI responsive.  Normally the function should return 0; returning a
'    non-zero value will cancel the current operation and cause HTTP() to return.
'    The callback function has one argument, the number of bytes received during the current iteration of
'    the receive loop.  Can be used for a progress bar etc.
'
'Output
'  header$ is the header of the downloaded file.
'  content$ is the content of the file.
'
'The RETURN value is
'     0 (no error),
'    -1 (error initializing Xin library),
'    -2 (error parsing URL),
'    -3 (error opening socket),
'    -4 (error connecting to server),
'    -5 (error sending HTTP request),
' or -6 (error reading received data).
'
' Using XinSetDebug() or ERROR() may give more information about the error.
'
FUNCTION  HTTP (URL$, referer$, callback, header$, content$)
	HOST server
	FUNCADDR  Callback(XLONG)
	STATIC init

	$serverPort	= 80
	$CRLF$ = "\r\n"
	$bufferSize = 8192

	content$ = ""
	header$ = ""
	Callback = callback

	error = -1
	IFZ init THEN GOSUB Initialize
	DEC error
	GOSUB ParseURL
	DEC error
	GOSUB OpenSocket
	DEC error
	GOSUB Connect
	DEC error
	GOSUB MakeRequest
	DEC error
	GOSUB Receive
	error = 0

CloseSocket:
	XinSocketClose (socket)
	RETURN (error)


'***************************
'********  Connect  ********
'***************************
'
SUB Connect
	IF XinHostNameToInfo (server$, @server) GOTO CloseSocket
	IF XinSocketConnectRequest (socket, 0, server.address, $serverPort) GOTO CloseSocket
	DO UNTIL connected
		IF XinSocketConnectStatus (socket, 100, @connected) GOTO CloseSocket
		IF Callback THEN
			ret = @Callback (0)
			IF ret THEN error = ret: GOTO CloseSocket
		END IF
	LOOP
END SUB

'***************************
'*****  ExtractChunk  ******
'***************************
'
' used only if the page headers include "Transfer-encoding: chunked"
SUB ExtractChunk
	IF chunkSize THEN
		IF chunkSize > LEN(buffer$) THEN
				content$ = content$ + buffer$
				chunkSize = chunkSize - LEN(buffer$)
				chunkSizePtr = chunkSizePtr - LEN(buffer$)
				EXIT SUB
			ELSE
				content$ = content$ + MID$(buffer$, 1, chunkSize)
				chunkSize = 0
		END IF
	END IF

	IF chunkSizePtr > LEN(buffer$) THEN
		chunkSizePtr = chunkSizePtr - LEN(buffer$)
		EXIT SUB
	END IF

	pcrlf = INSTR(buffer$, $CRLF$, chunkSizePtr)
	IFZ pcrlf THEN
		IF RIGHT$(buffer$,1) = "\r" THEN pcrlf = LEN(buffer$)
	END IF
	IFZ pcrlf THEN
			chunkSize$ = MID$(buffer$, chunkSizePtr)
			chunkSizePtr = 1
			EXIT SUB
		ELSE
			chunkSize$ = chunkSize$ + MID$(buffer$, chunkSizePtr, pcrlf - chunkSizePtr)
			chunkPtr = pcrlf + 2	'points to beginning of chunk data
			chunkSize = XLONG("0x" + chunkSize$)
			IFZ chunkSize THEN lastChunk = $$TRUE: EXIT SUB
			chunkSizePtr = chunkPtr + chunkSize + 2		'points to next chunk size entry
			chunkSize$ = ""
			IF chunkPtr > LEN(buffer$) THEN EXIT SUB
	END IF

	IF (chunkPtr + chunkSize - 1) <= LEN(buffer$) THEN
			content$ = content$ + MID$(buffer$, chunkPtr, chunkSize)
			chunkSize = 0
			GOSUB ExtractChunk
		ELSE
			content$ = content$ + MID$(buffer$, chunkPtr)
			chunkSize = chunkSize - (LEN(buffer$) - chunkPtr + 1)
			chunkSizePtr = chunkSizePtr - LEN(buffer$)
			chunkPtr = 1
	END IF
END SUB

'***************************
'*****  ExtractHeader  *****
'***************************
'
SUB ExtractHeader
	phdr = INSTR(buffer$, $CRLF$+$CRLF$)  'marks end of header
	IF phdr THEN
		header$ = header$ + LEFT$(buffer$, phdr+3)
		buffer$ = MID$(buffer$, phdr+4)
		gotHeader = $$TRUE
	ELSE  'header not yet completely downloaded
		header$ = header$ + buffer$
		EXIT SUB
	END IF

'need to know transfer-encoding and content-length, if specified
	XstStringToStringArray (@header$, @header$[])
	FOR k = 1 TO UBOUND (header$[])
		pcolon = INSTR(header$[k], ":")
		SELECT CASE TRUE
			CASE INSTRI(header$[k], "Content-Length:")
				contentLength = ULONG(MID$(header$[k], pcolon+1))
			CASE INSTRI(header$[k], "Transfer-Encoding:")
				chunked$ = LCASE$(TRIM$(MID$(header$[k], pcolon+1)))
				IF chunked$ == "chunked" THEN chunked = $$TRUE
		END SELECT
	NEXT k

'anything after the header is the beginning of page content
	IF chunked THEN
			chunkSize = 0
			chunkSizePtr = 1
			GOSUB ExtractChunk
		ELSE
			content$ = buffer$
	END IF
END SUB

'***************************
'******  Initialize  *******
'***************************
'
SUB Initialize
	IF XinInitialize (0, 0, 0, 0, "", "") THEN RETURN (error)
	init = $$TRUE
END SUB

'***************************
'******  MakeRequest  ******
'***************************
'
SUB MakeRequest
	request$ = "GET " + file$ + " HTTP/1.1" + $CRLF$
	request$ = request$ + "Host: " + server$ + $CRLF$
	request$ = request$ + "Accept: text/html, image/jpeg, */*" + $CRLF$
	IF referer$ THEN request$ = request$ + "Referer: " + referer$ + $CRLF$
	request$ = request$ + $CRLF$ 'double CRLF terminates request
	IF XinSocketWrite (socket, 100, &request$, LEN(request$), 0, @bytes) GOTO CloseSocket
END SUB

'***************************
'******  OpenSocket  *******
'***************************
'
SUB OpenSocket
	IF XinSocketOpen (@socket, 0, 0, 0)	GOTO CloseSocket
	IF XinSocketBind (socket, 0, 0, 0) GOTO CloseSocket
END SUB

'***************************
'*******  ParseURL  ********
'***************************
'
SUB ParseURL
	URL$ = TRIM$(URL$)
	p = INSTR(URL$, "://")
	IFZ p THEN
		protocol$ = "http://"
		url$ = URL$
	ELSE
		protocol$ = LCASE$(LEFT$(URL$, p+2))
		SELECT CASE protocol$
			CASE "http://"
			CASE ELSE	: RETURN (error)
		END SELECT
		url$ = MID$(URL$, p+3)
	END IF

	p = INSTR(url$,"/")
	IF p THEN
			server$ = LEFT$(url$, p-1)
			file$ = MID$(url$, p)
		ELSE
			server$ = url$
			file$ = "/"
	END IF
END SUB

'***************************
'********  Receive  ********
'***************************
'
SUB Receive
	DO
		buffer$ = NULL$($bufferSize)
		IF XinSocketRead (socket, 20000, &buffer$, $bufferSize, 0, @bytes) THEN EXIT DO
		IF bytes THEN
			buffer$ = LEFT$(buffer$, bytes)
			IF gotHeader THEN
				IF chunked THEN
					GOSUB ExtractChunk
					IF lastChunk THEN EXIT DO
				ELSE
					content$ = content$ + buffer$
				END IF
			ELSE
				GOSUB ExtractHeader
			END IF
		END IF
		IF contentLength && (LEN(content$) >= contentLength) THEN EXIT DO
		IF Callback THEN
			ret = @Callback (bytes)
			IF ret THEN error = ret: EXIT DO
		END IF
	LOOP
END SUB

END FUNCTION
'
'
' #########################
' #####  Blowback ()  #####
' #########################
'
FUNCTION  Blowback ()

'make sure socket is closed
	XinSocketClose (-1)

END FUNCTION
'
'
' #########################
' #####  Callback ()  #####
' #########################
'
FUNCTION  Callback (bytes)

'	? "bytes processed :" bytes

END FUNCTION
END PROGRAM
